/*
    Project: TCTEMP
    Version: V1.4a
    File   : tc.h

    Copyright (c) 2006 Author of TCTEMP. All rights reserved.

    Covered by TrueCrypt License 2.1 the full text of which is contained
    in the file License.txt included in TCTEMP binary and source code
    distribution archives.

    The copyright notice of this source code file and the following public key
    must be retained in full.

-----BEGIN PGP PUBLIC KEY BLOCK-----
Version: GnuPG v1.4.2.1 (MingW32)

mQGiBEP09h0RBAC1RkmI9S9JfZ0yP+i1j3GgOE+49a7vDCLmIRHEk8KfDR31d2iC
WsjvVWGPSkl0GWfJeWiIU2xOaFjo7zF0Os1nICG1x0hj2p6O3FRjBCPQEFjuXPmV
OexI9DiT/KtCvqjtWTnLwSWBCtH165ond/Aloe2TuNbiCWDoO5VyPNm6DwCg6QHF
bwnTp74YgYsuCVyMI1G5UeUD/AylpfON3ndCqgFgrZm4nJ93qnaSHP/fj/83yqDi
6xh339pqfzoyPMyP85q2QxNQ7PrOKfTHmsrXVj3zzfIz0VQxp7hflxzE6ydJaIzD
YpKZO5qxTRXvMqUxFo275MinDcIMeu2YUOWHnQgFQuZSuQAfsVGxSv4MFxPUQF+G
5/xCA/9wcVsBaGBXkaqVpMbeRx5+UUohEaEMuHPk0Lj6/rGlvAQpsVvuLZ3zO6C2
X7RXtsH8DfwqBAMhNkTqv9RmjUXsryGhrrDgxq7agjJPpbbIfgp/S3OYnwOx1BTb
OwqpTuTcqAJlDtCk3WOuMDa4h6DQeqb9NpIUM/lhkrgjIRqhJ7QQQXV0aG9yIG9m
IFRDVEVNUIhgBBMRAgAgBQJD9PYdAhsDBgsJCAcDAgQVAggDBBYCAwECHgECF4AA
CgkQSl/DkxnubmlEZACbBr6Fsvq5XGgxxHSiZ/DpoMDpxLIAn16QzZeLlLOjb7/I
TzCAdffqb4kW
=61fR
-----END PGP PUBLIC KEY BLOCK-----
*/

#ifndef __TC_H
#define __TC_H

#ifdef __cplusplus
extern "C" {
#endif

#include "tomcrypt.h"

#define TC_MAX_PASSWORD_LEN    64
#define TC_MAX_PASSWORD        (TC_MAX_PASSWORD_LEN + 1)

#define TC_HEADER_ID           0x54525545
#define TC_HEADER_VERSION      0x0002
#define TC_REQ_PROGRAM_VERSION 0x0410

#pragma pack(push, 1)

typedef unsigned __int8  TC_UINT8;
typedef unsigned __int16 TC_UINT16;
typedef unsigned __int32 TC_UINT32;
typedef unsigned __int64 TC_UINT64;

typedef struct
{
    /* not encrypted */
    TC_UINT8  rgbSalt[64];

    /* encrypted */
    TC_UINT32 dwId;
    TC_UINT16 wHeaderVersion;
    TC_UINT16 wReqProgramVersion;
    TC_UINT32 dwCRC32;
    TC_UINT64 ullVolumeCreationTime;
    TC_UINT64 ullHeaderModificationTime;
    TC_UINT64 ullHiddenVolumeSize;
    TC_UINT8  rgbReserved[156];
    TC_UINT8  rgbSecondaryKey[32];
    TC_UINT8  rgbMasterKey[96];
    TC_UINT8  rgbUnused[128];

} TC_HEADER, *PTC_HEADER;

typedef struct
{
    TC_UINT8  rgbSecondaryKey[32];
    TC_UINT8  rgbMasterKey[96];
} TC_KEY, *PTC_KEY;

typedef struct
{
    const struct ltc_cipher_descriptor *pdCipher[3];
    symmetric_key                      sk[3];
    TC_UINT8                           rgbSecondaryKey[16];

    /* temporary data */
    TC_UINT8                           rgbT[16];
    TC_UINT8                           rgbI[8];

} TCE_CONTEXT, *PTCE_CONTEXT;

#define TC_DEFAULT_ENTROPY_POOL_SIZE 64
typedef struct
{
    TC_UINT8  *pbWritePos;
    TC_UINT8  *pbPoolEnd;
    TC_UINT32 dwCRC;
    TC_UINT8  rgbPool[TC_DEFAULT_ENTROPY_POOL_SIZE];
} TC_ENTROPY_POOL, *PTC_ENTROPY_POOL;

typedef struct
{
    unsigned cbPasswordSize;
    char     rgbPassword[TC_MAX_PASSWORD];
} TC_PASSWORD, *PTC_PASSWORD;

typedef struct
{
    TC_UINT8    rgbFrequency[256];
    TC_PASSWORD pwTemp;
} TC_PASSWORD_INFO, *PTC_PASSWORD_INFO;

typedef struct
{
    union {
        PKCS5_2_BUFFER   pkcs5_2_state;
        TCE_CONTEXT      ctx;
        TC_PASSWORD_INFO pi;
    };
    TC_KEY      key;
} TC_BUFFER, *PTC_BUFFER;

#pragma pack(pop)

#define TC_AES                 1
#define TC_BLOWFISH            2
#define TC_CAST5               3
#define TC_SERPENT             4
#define TC_TRIPLEDES           5
#define TC_TWOFISH             6
#define TC_AES_TWOFISH         7
#define TC_AES_TWOFISH_SERPENT 8
#define TC_SERPENT_AES         9
#define TC_SERPENT_TWOFISH_AES 10
#define TC_TWOFISH_SERPENT     11

#define TC_RIPEMD160           1
#define TC_SHA1                2
#define TC_WHIRLPOOL           3

#define TC_RIPEMD160_ITERATION_COUNT 2000
#define TC_SHA1_ITERATION_COUNT      2000
#define TC_WHIRLPOOL_ITERATION_COUNT 1000

#ifndef TRUE
  typedef int BOOL;
# define TRUE  (1==1)
# define FALSE (1!=1)
#endif

#define TC_GetEntropyPoolSize(pep) ((pep)->pbPoolEnd - (pep)->rgbPool)
void __stdcall TC_InitEntropyPool(PTC_ENTROPY_POOL pep, size_t cbPoolSize);
void __stdcall TC_FillEntropyPool(PTC_ENTROPY_POOL pep, const void *pData,
  size_t cbDataSize);

BOOL __stdcall TC_CipherIsValid(unsigned uCipher);
unsigned __stdcall TC_GetCipherIndex(const char *paszCipherName);
const char * __stdcall TC_GetCipherName(unsigned uCipher);

BOOL __stdcall TC_HashIsValid(unsigned uHash);
unsigned __stdcall TC_GetHashIndex(const char *paszHashName);
const char * __stdcall TC_GetHashName(unsigned uHash);
unsigned __stdcall TC_GetDefaultIterationCount(unsigned uHash);

BOOL __stdcall TC_KeyIsWeak(PTC_KEY pKey, unsigned uCipher);
BOOL __stdcall TC_PasswordIsWeak(PTC_PASSWORD pPassword,
  unsigned uDesiredStrength, PTC_PASSWORD_INFO);

BOOL __stdcall TCE_Init(PTCE_CONTEXT pCtx, PTC_KEY pKey, unsigned uCipher);
BOOL __stdcall TCE_SetIndex(PTCE_CONTEXT pCtx, unsigned __int64 ullIndex);
BOOL __stdcall TCE_SetSectorNumber(PTCE_CONTEXT pCtx,
  unsigned __int64 ullSectorNumber);
BOOL __stdcall TCE_Encrypt(PTCE_CONTEXT pCtx, void *pDest, size_t cbDestSize);
BOOL __stdcall TCE_Decrypt(PTCE_CONTEXT pCtx, void *pDest, size_t cbDestSize);
void __stdcall TCE_Close(PTCE_CONTEXT pCtx);

BOOL __stdcall TC_DeriveKey(void *pKey, size_t cbKeySize,
  const void *pPassword, size_t cbPasswordSize,
  const void *pSalt, size_t cbSaltSize,
  unsigned uHash, unsigned uIterationCount,
  PPKCS5_2_BUFFER pBuffer);
BOOL __stdcall TC_DeriveNonWeakKey(PTC_KEY pKey,
  const void *pPassword, size_t cbPasswordSize,
  void *pSalt, size_t cbSaltSize,
  unsigned uHash, unsigned uIterationCount,
  unsigned uCipher,
  PPKCS5_2_BUFFER pBuffer);
BOOL __stdcall TC_EncryptHeader(PTC_HEADER pHeader, const void *pPassword,
  size_t cbPasswordSize, unsigned uHash, unsigned uCipher, PTC_BUFFER pBuffer);
BOOL TC_MakeRandomHeader(PTC_HEADER pHeader, PTC_PASSWORD pPassword,
  PTC_ENTROPY_POOL pEntropyPool, unsigned uHash, unsigned uCipher,
  unsigned uIterationCount, PTC_BUFFER pBuffer);

#ifdef __cplusplus
}
#endif

#endif
