/*
    Project: TCTEMP
    Version: V1.4a
    File   : tc_algidx.c

    Copyright (c) 2006 Author of TCTEMP. All rights reserved.

    Covered by TrueCrypt License 2.1 the full text of which is contained
    in the file License.txt included in TCTEMP binary and source code
    distribution archives.

    The copyright notice of this source code file and the following public key
    must be retained in full.

-----BEGIN PGP PUBLIC KEY BLOCK-----
Version: GnuPG v1.4.2.1 (MingW32)

mQGiBEP09h0RBAC1RkmI9S9JfZ0yP+i1j3GgOE+49a7vDCLmIRHEk8KfDR31d2iC
WsjvVWGPSkl0GWfJeWiIU2xOaFjo7zF0Os1nICG1x0hj2p6O3FRjBCPQEFjuXPmV
OexI9DiT/KtCvqjtWTnLwSWBCtH165ond/Aloe2TuNbiCWDoO5VyPNm6DwCg6QHF
bwnTp74YgYsuCVyMI1G5UeUD/AylpfON3ndCqgFgrZm4nJ93qnaSHP/fj/83yqDi
6xh339pqfzoyPMyP85q2QxNQ7PrOKfTHmsrXVj3zzfIz0VQxp7hflxzE6ydJaIzD
YpKZO5qxTRXvMqUxFo275MinDcIMeu2YUOWHnQgFQuZSuQAfsVGxSv4MFxPUQF+G
5/xCA/9wcVsBaGBXkaqVpMbeRx5+UUohEaEMuHPk0Lj6/rGlvAQpsVvuLZ3zO6C2
X7RXtsH8DfwqBAMhNkTqv9RmjUXsryGhrrDgxq7agjJPpbbIfgp/S3OYnwOx1BTb
OwqpTuTcqAJlDtCk3WOuMDa4h6DQeqb9NpIUM/lhkrgjIRqhJ7QQQXV0aG9yIG9m
IFRDVEVNUIhgBBMRAgAgBQJD9PYdAhsDBgsJCAcDAgQVAggDBBYCAwECHgECF4AA
CgkQSl/DkxnubmlEZACbBr6Fsvq5XGgxxHSiZ/DpoMDpxLIAn16QzZeLlLOjb7/I
TzCAdffqb4kW
=61fR
-----END PGP PUBLIC KEY BLOCK-----
*/

#include "tc.h"

typedef struct {
    unsigned   uIndex;
    const char *paszName;
} CIPHER_NAME, *PCIPHER_NAME;

static CIPHER_NAME cnCiphers[] = {
#ifdef RIJNDAEL
    { TC_AES, "AES" },
    { TC_AES, "AES256" },
    { TC_AES, "Rijndael" },
#endif
#ifdef BLOWFISH
    { TC_BLOWFISH, "Blowfish" },
#endif
#ifdef CAST5
    { TC_CAST5, "CAST5" },
#endif
#ifdef SERPENT
    { TC_SERPENT, "Serpent" },
#endif
#ifdef DES
    { TC_TRIPLEDES, "Triple DES" },
    { TC_TRIPLEDES, "TripleDES" },
    { TC_TRIPLEDES, "3DES" },
#endif
#ifdef TWOFISH
    { TC_TWOFISH, "Twofish" },
#endif
#if defined(RIJNDAEL) && defined(TWOFISH)
    { TC_AES_TWOFISH, "AES-Twofish" },
#endif
#if defined(RIJNDAEL) && defined(TWOFISH) && defined(SERPENT)
    { TC_AES_TWOFISH_SERPENT, "AES-Twofish-Serpent" },
#endif
#if defined(SERPENT) && defined(RIJNDAEL)
    { TC_SERPENT_AES, "Serpent-AES" },
#endif
#if defined(SERPENT) && defined(TWOFISH) && defined(RIJNDAEL)
    { TC_SERPENT_TWOFISH_AES, "Serpent-Twofish-AES" },
#endif
#if defined(TWOFISH) && defined(SERPENT)
    { TC_TWOFISH_SERPENT, "Twofish-Serpent" },
#endif
    { 0, NULL }
};

typedef struct {
    unsigned   uIndex;
    unsigned   uDefaultIterationCount;
    const char *paszName;
} HASH_NAME, *PHASH_NAME;

static HASH_NAME hnHashes[] = {
#ifdef RIPEMD160
    { TC_RIPEMD160, TC_RIPEMD160_ITERATION_COUNT, "RIPEMD-160" },
    { TC_RIPEMD160, TC_RIPEMD160_ITERATION_COUNT, "RIPEMD160" },
#endif
#ifdef SHA1
    { TC_SHA1, TC_SHA1_ITERATION_COUNT, "SHA-1" },
    { TC_SHA1, TC_SHA1_ITERATION_COUNT, "SHA1" },
#endif
#ifdef WHIRLPOOL
    { TC_WHIRLPOOL, TC_WHIRLPOOL_ITERATION_COUNT, "Whirlpool" },
#endif
    { 0, 0, NULL }
};

BOOL __stdcall TC_CipherIsValid(unsigned uCipher)
{
    PCIPHER_NAME p = cnCiphers;

    while (p->uIndex) {
        if (p->uIndex == uCipher) {
            return TRUE;
        }
        p++;
    }

    return FALSE;
}

unsigned __stdcall TC_GetCipherIndex(const char *paszCipherName)
{
    PCIPHER_NAME p = cnCiphers;

    while (p->uIndex) {
        if (stricmp(paszCipherName, p->paszName) == 0) {
            return p->uIndex;
        }
        p++;
    }

    return 0;
}

const char * __stdcall TC_GetCipherName(unsigned uCipher)
{
    PCIPHER_NAME p = cnCiphers;

    while (p->uIndex) {
        if (p->uIndex == uCipher) {
            return p->paszName;
        }
        p++;
    }

    return NULL;
}

BOOL __stdcall TC_HashIsValid(unsigned uHash)
{
    PHASH_NAME p = hnHashes;

    while (p->uIndex) {
        if (p->uIndex == uHash) {
            return TRUE;
        }
        p++;
    }

    return FALSE;
}

unsigned __stdcall TC_GetHashIndex(const char *paszHashName)
{
    PHASH_NAME p = hnHashes;

    while (p->uIndex) {
        if (stricmp(paszHashName, p->paszName) == 0) {
            return p->uIndex;
        }
        p++;
    }

    return 0;
}

const char * __stdcall TC_GetHashName(unsigned uHash)
{
    PHASH_NAME p = hnHashes;

    while (p->uIndex) {
        if (p->uIndex == uHash) {
            return p->paszName;
        }
        p++;
    }

    return NULL;
}

unsigned __stdcall TC_GetDefaultIterationCount(unsigned uHash)
{
    PHASH_NAME p = hnHashes;

    while (p->uIndex) {
        if (p->uIndex == uHash) {
            return p->uDefaultIterationCount;
        }
        p++;
    }

    return 0;
}