/*
    Project: TCTEMP
    Version: V1.4a
    File   : tc_entropy.c

    Copyright (c) 2006 Author of TCTEMP. All rights reserved.

    Covered by TrueCrypt License 2.1 the full text of which is contained
    in the file License.txt included in TCTEMP binary and source code
    distribution archives.

    The copyright notice of this source code file and the following public key
    must be retained in full.

-----BEGIN PGP PUBLIC KEY BLOCK-----
Version: GnuPG v1.4.2.1 (MingW32)

mQGiBEP09h0RBAC1RkmI9S9JfZ0yP+i1j3GgOE+49a7vDCLmIRHEk8KfDR31d2iC
WsjvVWGPSkl0GWfJeWiIU2xOaFjo7zF0Os1nICG1x0hj2p6O3FRjBCPQEFjuXPmV
OexI9DiT/KtCvqjtWTnLwSWBCtH165ond/Aloe2TuNbiCWDoO5VyPNm6DwCg6QHF
bwnTp74YgYsuCVyMI1G5UeUD/AylpfON3ndCqgFgrZm4nJ93qnaSHP/fj/83yqDi
6xh339pqfzoyPMyP85q2QxNQ7PrOKfTHmsrXVj3zzfIz0VQxp7hflxzE6ydJaIzD
YpKZO5qxTRXvMqUxFo275MinDcIMeu2YUOWHnQgFQuZSuQAfsVGxSv4MFxPUQF+G
5/xCA/9wcVsBaGBXkaqVpMbeRx5+UUohEaEMuHPk0Lj6/rGlvAQpsVvuLZ3zO6C2
X7RXtsH8DfwqBAMhNkTqv9RmjUXsryGhrrDgxq7agjJPpbbIfgp/S3OYnwOx1BTb
OwqpTuTcqAJlDtCk3WOuMDa4h6DQeqb9NpIUM/lhkrgjIRqhJ7QQQXV0aG9yIG9m
IFRDVEVNUIhgBBMRAgAgBQJD9PYdAhsDBgsJCAcDAgQVAggDBBYCAwECHgECF4AA
CgkQSl/DkxnubmlEZACbBr6Fsvq5XGgxxHSiZ/DpoMDpxLIAn16QzZeLlLOjb7/I
TzCAdffqb4kW
=61fR
-----END PGP PUBLIC KEY BLOCK-----
*/

#include "tc_priv.h"

/*
 * Pool filling method taken from TrueCrypt keyfile applying algorithm
 */

void __stdcall TC_InitEntropyPool(PTC_ENTROPY_POOL pep, size_t cbPoolSize)
{
    TC_ASSERT(pep);
    TC_ASSERT((cbPoolSize & 3) == 0);

    if (cbPoolSize == 0) {
        cbPoolSize = sizeof(pep->rgbPool);
    }

    pep->pbWritePos = pep->rgbPool;
    pep->pbPoolEnd  = pep->rgbPool + cbPoolSize;
}

void __stdcall TC_FillEntropyPool(PTC_ENTROPY_POOL pep,
  const void *pData, size_t cbDataSize)
{
    const unsigned __int8 *pbData;
    const unsigned __int8 *pbDataEnd;

    TC_ASSERT(pep);
    TC_ASSERT(pData);

    pbData    = (const unsigned __int8 *) pData;
    pbDataEnd = pbData + cbDataSize;

    for (; pbData != pbDataEnd; pbData++) {
        UpdateCRC(pep->dwCRC, *pbData);
        *pep->pbWritePos++ += ((const unsigned __int8 *)&pep->dwCRC)[3];
        *pep->pbWritePos++ += ((const unsigned __int8 *)&pep->dwCRC)[2];
        *pep->pbWritePos++ += ((const unsigned __int8 *)&pep->dwCRC)[1];
        *pep->pbWritePos++ += ((const unsigned __int8 *)&pep->dwCRC)[0];
        if (pep->pbWritePos == pep->pbPoolEnd) {
            pep->pbWritePos = pep->rgbPool;
        }
    }
}
