/*
    Project: TCTEMP
    Version: V1.4a
    File   : tc_random_header.c

    Copyright (c) 2006 Author of TCTEMP. All rights reserved.

    Covered by TrueCrypt License 2.1 the full text of which is contained
    in the file License.txt included in TCTEMP binary and source code
    distribution archives.

    The copyright notice of this source code file and the following public key
    must be retained in full.

-----BEGIN PGP PUBLIC KEY BLOCK-----
Version: GnuPG v1.4.2.1 (MingW32)

mQGiBEP09h0RBAC1RkmI9S9JfZ0yP+i1j3GgOE+49a7vDCLmIRHEk8KfDR31d2iC
WsjvVWGPSkl0GWfJeWiIU2xOaFjo7zF0Os1nICG1x0hj2p6O3FRjBCPQEFjuXPmV
OexI9DiT/KtCvqjtWTnLwSWBCtH165ond/Aloe2TuNbiCWDoO5VyPNm6DwCg6QHF
bwnTp74YgYsuCVyMI1G5UeUD/AylpfON3ndCqgFgrZm4nJ93qnaSHP/fj/83yqDi
6xh339pqfzoyPMyP85q2QxNQ7PrOKfTHmsrXVj3zzfIz0VQxp7hflxzE6ydJaIzD
YpKZO5qxTRXvMqUxFo275MinDcIMeu2YUOWHnQgFQuZSuQAfsVGxSv4MFxPUQF+G
5/xCA/9wcVsBaGBXkaqVpMbeRx5+UUohEaEMuHPk0Lj6/rGlvAQpsVvuLZ3zO6C2
X7RXtsH8DfwqBAMhNkTqv9RmjUXsryGhrrDgxq7agjJPpbbIfgp/S3OYnwOx1BTb
OwqpTuTcqAJlDtCk3WOuMDa4h6DQeqb9NpIUM/lhkrgjIRqhJ7QQQXV0aG9yIG9m
IFRDVEVNUIhgBBMRAgAgBQJD9PYdAhsDBgsJCAcDAgQVAggDBBYCAwECHgECF4AA
CgkQSl/DkxnubmlEZACbBr6Fsvq5XGgxxHSiZ/DpoMDpxLIAn16QzZeLlLOjb7/I
TzCAdffqb4kW
=61fR
-----END PGP PUBLIC KEY BLOCK-----
*/

#include "tc_priv.h"

//#define USE_ASCII_PASSWORD

#pragma pack(push, 1)

typedef struct
{
    TC_UINT8 rgbSalt[64];
    TC_UINT8 rgbPasswordSalt[64];
    TC_UINT8 rgbReserved[128];
    TC_KEY   keyVolume; /* 128 bytes */
    TC_UINT8 rgbPassword[64];
    TC_UINT8 rgbUnused[64];
} TC_OLD_HEADER, *PTC_OLD_HEADER;

#pragma pack(pop)

BOOL TC_MakeRandomHeader(PTC_HEADER pHeader, PTC_PASSWORD pPassword,
  PTC_ENTROPY_POOL pEntropyPool, unsigned uHash, unsigned uCipher,
  unsigned uIterationCount, PTC_BUFFER pBuffer)
{
    PTC_OLD_HEADER poh;
    unsigned       i;
#ifdef USE_ASCII_PASSWORD
    unsigned char  *pb;
    unsigned char  *pbEnd;
#endif

    poh = (PTC_OLD_HEADER)pHeader;

    if (!TC_HashIsValid(uHash) || !TC_CipherIsValid(uCipher)) {
        return FALSE;
    }

    pPassword->rgbPassword[TC_MAX_PASSWORD_LEN] = '\0';
    pPassword->cbPasswordSize = TC_MAX_PASSWORD_LEN;
    for (i = 256; --i; ) {
        if (!TC_DeriveKey(&poh->keyVolume,
          sizeof(TC_KEY)+sizeof(poh->rgbPassword), pEntropyPool->rgbPool,
          TC_GetEntropyPoolSize(pEntropyPool), poh->rgbPasswordSalt,
          sizeof(poh->rgbPasswordSalt), uHash, uIterationCount,
          &pBuffer->pkcs5_2_state)) {
            return FALSE;
        }
        memcpy(pPassword->rgbPassword, poh->rgbPassword, TC_MAX_PASSWORD_LEN);
#ifdef USE_ASCII_PASSWORD
        pb = pPassword->rgbPassword;
        pbEnd = pb + TC_MAX_PASSWORD_LEN;
        for (; pb != pbEnd; pb++) {
            *pb = '0' + (*pb & 0x3f);
        }
#endif
        if (!TC_KeyIsWeak(&poh->keyVolume, uCipher) &&
          !TC_PasswordIsWeak(pPassword, 256, &pBuffer->pi)) {
            break;
        }
        poh->rgbPasswordSalt[0]++;
    }
    if (i == 0) {
        return FALSE;
    }

    memset(poh->rgbSalt, '\0', 256);
    memset(poh->rgbPassword, '\0', 128);
    if (TCE_Init(&pBuffer->ctx, &poh->keyVolume, uCipher)) {
        TCE_SetIndex(&pBuffer->ctx, CONST64(0xfffffffffffffff8));
        if (!TCE_Encrypt(&pBuffer->ctx, poh->rgbSalt, 64)) {
            TCE_Close(&pBuffer->ctx);
            return FALSE;
        }
        TCE_Close(&pBuffer->ctx);
    }

    return TC_EncryptHeader(pHeader, pPassword->rgbPassword,
      TC_MAX_PASSWORD_LEN, uHash, uCipher, pBuffer);
}
